!******************************************************************************
!****m* /besselK
! NAME
! module besselK
! PURPOSE
! provide the 'modified Bessel functions of first kind', I_n, and the
! 'modified Bessel functions of second kind', K_n.
!
! NOTES
! * The expansions by Pavel Holoborodko are used,
!   https://www.advanpix.com/2015/11/11/rational-approximations-for-the-modified-bessel-function-of-the-first-kind-i0-computations-double-precision/
!   (see for further references below at the definition of the fuction.)
!   These expressions claim to be exact within one unit of machine precision,
!   i.e. ~2e-16
! * The series expansions given by Abramowitz&Stegun (chapter 9.8)
!   only have single precission (~1e-7)
! * K0 and K1 are imlemented explicitely, using own parametrization of I0 and
!   I1. K2 is implemented by recursion.
! * One should catch the case of x > 400, since then the exp results as NaN.
! * One could think of implementing a second order Horner scheme in order
!   to speed up the calculations of the polynomials.
!   Here one splits the polynomial p(x) into the odd and even polynomials
!   p_odd(y) and p_even(y) (with y=x^2) and returns p(x) = p_even + x*p_odd.
!   see e.g.:
!      J.-M. Muller, Elementary Functions. Algorithms and Implementation
!      Birkhäuser / Springer, 2016
!******************************************************************************
module besselK
  implicit none
  private

  public :: BesselI0
  public :: BesselI1
  public :: BesselK0
  public :: BesselK1
  public :: BesselK2

  public :: BesselKexp0
  public :: BesselKexp1
  public :: BesselKexp2


  ! Mathematica:
  ! SS = Series[...]
  ! FortranForm[SetPrecision[HornerForm[Normal[SS], x], 20]]

contains
  !****************************************************************************
  !****f* besselK/BesselI0
  ! NAME
  ! real function BesselI0(x)
  ! PURPOSE
  ! return the value I(0,x)
  !
  ! NOTES
  ! https://www.advanpix.com/2015/11/11/rational-approximations-for-the-modified-bessel-function-of-the-first-kind-i0-computations-double-precision/
  !****************************************************************************
  elemental real function BesselI0(x)
    real, intent(in) :: x

    real :: t

    if (x < 0.) then
       BesselI0 = 0

    else if (x < 7.75) then

       t = (x/2)**2

       BesselI0 = 1.0000000000000000000000801e+00 &
            + t*(2.4999999999999999999629693e-01 &
            + t*(2.7777777777777777805664954e-02 &
            + t*(1.7361111111111110294015271e-03 &
            + t*(6.9444444444444568581891535e-05 &
            + t*(1.9290123456788994104574754e-06 &
            + t*(3.9367598891475388547279760e-08 &
            + t*(6.1511873265092916275099070e-10 &
            + t*(7.5940584360755226536109511e-12 &
            + t*(7.5940582595094190098755663e-14 &
            + t*(6.2760839879536225394314453e-16 &
            + t*(4.3583591008893599099577755e-18 &
            + t*(2.5791926805873898803749321e-20 &
            + t*(1.3141332422663039834197910e-22 &
            + t*(5.9203280572170548134753422e-25 &
            + t*(2.0732014503197852176921968e-27 &
            + t*(1.1497640034400735733456400e-29 &
            ))))))))))))))))

       BesselI0 = BesselI0 * t + 1.0

    else
       t = 1/x

       BesselI0 = 3.9894228040143265335649948e-01 &
            + t*(4.9867785050353992900698488e-02 &
            + t*(2.8050628884163787533196746e-02 &
            + t*(2.9219501690198775910219311e-02 &
            + t*(4.4718622769244715693031735e-02 &
            + t*(9.4085204199017869159183831e-02 &
            + t*(-1.0699095472110916094973951e-01 &
            + t*(2.2725199603010833194037016e+01 &
            + t*(-1.0026890180180668595066918e+03 &
            + t*(3.1275740782277570164423916e+04 &
            + t*(-5.9355022509673600842060002e+05 &
            + t*(2.6092888649549172879282592e+06 &
            + t*(2.3518420447411254516178388e+08 &
            + t*(-8.9270060370015930749184222e+09 &
            + t*(1.8592340458074104721496236e+11 &
            + t*(-2.6632742974569782078420204e+12 &
            + t*(2.7752144774934763122129261e+13 &
            + t*(-2.1323049786724612220362154e+14 &
            + t*(1.1989242681178569338129044e+15 &
            + t*(-4.8049082153027457378879746e+15 &
            + t*(1.3012646806421079076251950e+16 &
            + t*(-2.1363029690365351606041265e+16 &
            + t*(1.6069467093441596329340754e+16 &
            ))))))))))))))))))))))

       BesselI0 = BesselI0 * exp(x)/sqrt(x)
    end if

  end function BesselI0

  !****************************************************************************
  !****f* besselK/BesselI1
  ! NAME
  ! real function BesselI1(x)
  ! PURPOSE
  ! return the value I(1,x)
  !
  ! NOTES
  ! https://www.advanpix.com/2015/11/12/rational-approximations-for-the-modified-bessel-function-of-the-first-kind-i1-for-computations-with-double-precision/
  !****************************************************************************
  elemental real function BesselI1(x)
    real, intent(in) :: x

    real :: t

    if (x < 0) then
       BesselI1 = 0.0

    else if (x < 7.75) then

       t = (x/2)**2

       BesselI1 = 8.3333333333333333311567967e-02 &
            + t*(6.9444444444444450632369337e-03 &
            + t*(3.4722222222221933634809047e-04 &
            + t*(1.1574074074079326676719210e-05 &
            + t*(2.7557319223490964726712181e-07 &
            + t*(4.9209498642000488498034902e-09 &
            + t*(6.8346524852208360284643288e-11 &
            + t*(7.5940608652484019265663823e-13 &
            + t*(6.9036483611746228414130722e-15 &
            + t*(5.2305536429160706017626195e-17 &
            + t*(3.3486060280590464196327437e-19 &
            + t*(1.8645262719811753663834319e-21 &
            + t*(7.9611250107842314599760659e-24 &
            + t*(5.3251032089995165438568695e-26 &
            )))))))))))))

       BesselI1 = (1.0 + t*(0.5 + t*BesselI1))*x/2

    else
       t = 1/x

       BesselI1 = 3.9894228040143270388374079e-01 &
            + t*(-1.4960335515072058522575487e-01 &
            + t*(-4.6751048269476797374239762e-02 &
            + t*(-4.0907267094886972971863462e-02 &
            + t*(-5.7501487840859800117669379e-02 &
            + t*(-1.1428156617865937773864845e-01 &
            + t*(6.7988447242260666801129937e-02 &
            + t*(-2.2694203870019250176636896e+01 &
            + t*(9.7548286270114208672947525e+02 &
            + t*(-2.9286459257939415083570152e+04 &
            + t*(4.9934855620495985742805154e+05 &
            + t*(5.7682364160056137069002930e+05 &
            + t*(-3.1576840778898356890175020e+08 &
            + t*(1.0484906321376589515223174e+10 &
            + t*(-2.0918193917759394367113655e+11 &
            + t*(2.9320804098307168426392082e+12 &
            + t*(-3.0147278411132255281401004e+13 &
            + t*(2.2950466603697814797615042e+14 &
            + t*(-1.2816007548999035598180100e+15 &
            + t*(5.1086996139908353110844064e+15 &
            + t*(-1.3774917783425787550429723e+16 &
            + t*(2.2531580094188348024267027e+16 &
            + t*(-1.6895178303473738478791245e+16 &
            ))))))))))))))))))))))

       BesselI1 = BesselI1 * exp(x)/sqrt(x)
    end if

  end function BesselI1


  !****************************************************************************
  !****f* besselK/BesselK0
  ! NAME
  ! real function BesselK0(x)
  ! PURPOSE
  ! return the value K(0,x)
  !
  ! NOTES
  ! https://www.advanpix.com/2015/11/25/rational-approximations-for-the-modified-bessel-function-of-the-second-kind-k0-for-computations-with-double-precision/
  !****************************************************************************
  elemental real function BesselK0(x)
    real, intent(in) :: x

    real :: t, I0

    if (x <= 0.0) then
       BesselK0 = 0.0

    else if (x < 1.0) then
       t = (x)**2

       BesselK0 = 1.1593151565841244842077226e-01 &
            + t*(2.7898287891460317300886539e-01 &
            + t*(2.5248929932161220559969776e-02 &
            + t*(8.4603509072136578707676406e-04 &
            + t*(1.4914719243067801775856150e-05 &
            + t*(1.6271068931224552553548933e-07 &
            + t*(1.2082660336282566759313543e-09 &
            + t*(6.6117104672254184399933971e-12 &
            )))))))

       t = (x/2)**2
       I0 = 1.0000000000000000044974165e+00 &
            + t*(2.4999999999999822316775454e-01 &
            + t*(2.7777777777892149148858521e-02 &
            + t*(1.7361111083544590676709592e-03 &
            + t*(6.9444476047072424198677755e-05 &
            + t*(1.9288265756466775034067979e-06 &
            + t*(3.9908220583262192851839992e-08 &
            ))))))

       BesselK0 = BesselK0 - log(x) * (t*I0 + 1.0)

    else

       t = 1/x

       BesselK0 = 1.0694678222191263215918328e-01 &
            + t*(9.0753360415683846760792445e-01 &
            + t*(1.7215172959695072045669045e+00 &
            + t*(-1.7172089076875257095489749e-01 &
            + t*(7.3154750356991229825958019e-02 &
            + t*(-5.4975286232097852780866385e-02 &
            + t*(5.7217703802970844746230694e-02 &
            + t*(-7.2884177844363453190380429e-02 &
            + t*(1.0443967655783544973080767e-01 &
            + t*(-1.5741597553317349976818516e-01 &
            + t*(2.3582486699296814538802637e-01 &
            + t*(-3.3484166783257765115562496e-01 &
            + t*(4.3328524890855568555069622e-01 &
            + t*(-4.9470375304462431447923425e-01 &
            + t*(4.8474122247422388055091847e-01 &
            + t*(-3.9725799556374477699937953e-01 &
            + t*(2.6507653322930767914034592e-01 &
            + t*(-1.3951265948137254924254912e-01 &
            + t*(5.5500667358490463548729700e-02 &
            + t*(-1.5636955694760495736676521e-02 &
            + t*(2.7741514506299244078981715e-03 &
            + t*(-2.3261089001545715929104236e-04 &
            )))))))))))))))))))))

       BesselK0 = BesselK0 / ( &
            8.5331186362410449871043129e-02 &
            + t*(7.3477344946182065340442326e-01 &
            + t*(1.4594189037511445958046540e+00 &
            ))) * exp(-x)/sqrt(x)

    end if

  end function BesselK0

  !****************************************************************************
  !****f* besselK/BesselK1
  ! NAME
  ! real function BesselK1(x)
  ! PURPOSE
  ! return the value K(1,x)
  !
  ! NOTES
  ! http://www.advanpix.com/2016/01/05/rational-approximations-for-the-modified-bessel-function-of-the-second-kind-k1-for-computations-with-double-precision/
  !****************************************************************************
  elemental real function BesselK1(x)
    real, intent(in) :: x

    real :: t, I1

    if (x <= 0.0) then
       BesselK1 = 0.0

    else if (x < 1.0) then
       t = (x)**2

       BesselK1 = -3.0796575782920622440538935e-01 &
            + t*(-8.5370719728650778045782736e-02 &
            + t*(-4.6421827664715603298154971e-03 &
            + t*(-1.1253607036630425931072996e-04 &
            + t*(-1.5592887702110907110292728e-06 &
            + t*(-1.4030163679125934402498239e-08 &
            + t*(-8.8718998640336832196558868e-11 &
            + t*(-4.1614323580221539328960335e-13 &
            + t*(-1.5261293392975541707230366e-15 &
            ))))))))

       t = (x/2)**2
       I1 = 8.3333333333333325191635191e-02 &
            + t*(6.9444444444467956461838830e-03 &
            + t*(3.4722222211230452695165215e-04 &
            + t*(1.1574075952009842696580084e-05 &
            + t*(2.7555870002088181016676934e-07 &
            + t*(4.9724386164128529514040614e-09 &
            )))))

       I1 = (1.0 + t*(0.5 + t*I1)) * x/2

       BesselK1 = x*BesselK1 + 1/x + log(x)*I1

    else

       t = 1/x

       BesselK1 = 1.0234817795732426171122752e-01 &
            + t*(9.4576473594736724815742878e-01 &
            + t*(2.1876721356881381470401990e+00 &
            + t*(6.0143447861316538915034873e-01 &
            + t*(-1.3961391456741388991743381e-01 &
            + t*(8.8229427272346799004782764e-02 &
            + t*(-8.5494054051512748665954180e-02 &
            + t*(1.0617946033429943924055318e-01 &
            + t*(-1.5284482951051872048173726e-01 &
            + t*(2.3707700686462639842005570e-01 &
            + t*(-3.7345723872158017497895685e-01 &
            + t*(5.6874783855986054797640277e-01 &
            + t*(-8.0418742944483208700659463e-01 &
            + t*(1.0215105768084562101457969e+00 &
            + t*(-1.1342221242815914077805587e+00 &
            + t*(1.0746932686976675016706662e+00 &
            + t*(-8.4904532475797772009120500e-01 &
            + t*(5.4542251056566299656460363e-01 &
            + t*(-2.7630896752209862007904214e-01 &
            + t*(1.0585982409547307546052147e-01 &
            + t*(-2.8751691985417886721803220e-02 &
            + t*(4.9233441525877381700355793e-03 &
            + t*(-3.9900679319457222207987456e-04 &
            ))))))))))))))))))))))

       BesselK1 = BesselK1 / ( &
            + 8.1662031018453173425764707e-02 &
            + t*(7.2398781933228355889996920e-01 &
            + t*(1.4835841581744134589980018e+00 &
            )))  * exp(-x)/sqrt(x)

    end if

  end function BesselK1

  !****************************************************************************
  !****f* besselK/BesselK2
  ! NAME
  ! real function BesselK2(x)
  ! PURPOSE
  ! return the value K(2,x)
  !****************************************************************************
  elemental real function BesselK2(x)
    real, intent(in) :: x

    if (x <= 0.0) then
       BesselK2 = 0.0
    else
       BesselK2 = BesselK0(x) + (2.0/x) * BesselK1(x)
    end if

  end function BesselK2

  !****************************************************************************
  !****f* besselK/BesselKexp0
  ! NAME
  ! real function BesselKexp0(x)
  ! PURPOSE
  ! return the value exp(x)*K(0,x)
  !
  ! NOTES
  ! https://www.advanpix.com/2015/11/25/rational-approximations-for-the-modified-bessel-function-of-the-second-kind-k0-for-computations-with-double-precision/
  !****************************************************************************
  elemental real function BesselKexp0(x)
    real, intent(in) :: x

    real :: t, I0

    if (x <= 0.0) then
       BesselKexp0 = 0.0

    else if (x < 1.0) then
       t = (x)**2

       BesselKexp0 = 1.1593151565841244842077226e-01 &
            + t*(2.7898287891460317300886539e-01 &
            + t*(2.5248929932161220559969776e-02 &
            + t*(8.4603509072136578707676406e-04 &
            + t*(1.4914719243067801775856150e-05 &
            + t*(1.6271068931224552553548933e-07 &
            + t*(1.2082660336282566759313543e-09 &
            + t*(6.6117104672254184399933971e-12 &
            )))))))

       t = (x/2)**2
       I0 = 1.0000000000000000044974165e+00 &
            + t*(2.4999999999999822316775454e-01 &
            + t*(2.7777777777892149148858521e-02 &
            + t*(1.7361111083544590676709592e-03 &
            + t*(6.9444476047072424198677755e-05 &
            + t*(1.9288265756466775034067979e-06 &
            + t*(3.9908220583262192851839992e-08 &
            ))))))

       BesselKexp0 = (BesselKexp0 - log(x) * (t*I0 + 1.0))*exp(x)

    else

       t = 1/x

       BesselKexp0 = 1.0694678222191263215918328e-01 &
            + t*(9.0753360415683846760792445e-01 &
            + t*(1.7215172959695072045669045e+00 &
            + t*(-1.7172089076875257095489749e-01 &
            + t*(7.3154750356991229825958019e-02 &
            + t*(-5.4975286232097852780866385e-02 &
            + t*(5.7217703802970844746230694e-02 &
            + t*(-7.2884177844363453190380429e-02 &
            + t*(1.0443967655783544973080767e-01 &
            + t*(-1.5741597553317349976818516e-01 &
            + t*(2.3582486699296814538802637e-01 &
            + t*(-3.3484166783257765115562496e-01 &
            + t*(4.3328524890855568555069622e-01 &
            + t*(-4.9470375304462431447923425e-01 &
            + t*(4.8474122247422388055091847e-01 &
            + t*(-3.9725799556374477699937953e-01 &
            + t*(2.6507653322930767914034592e-01 &
            + t*(-1.3951265948137254924254912e-01 &
            + t*(5.5500667358490463548729700e-02 &
            + t*(-1.5636955694760495736676521e-02 &
            + t*(2.7741514506299244078981715e-03 &
            + t*(-2.3261089001545715929104236e-04 &
            )))))))))))))))))))))

       BesselKexp0 = BesselKexp0 / ( &
            8.5331186362410449871043129e-02 &
            + t*(7.3477344946182065340442326e-01 &
            + t*(1.4594189037511445958046540e+00 &
            ))) / sqrt(x)

    end if

  end function BesselKexp0

  !****************************************************************************
  !****f* besselK/BesselKexp1
  ! NAME
  ! real function BesselKexp1(x)
  ! PURPOSE
  ! return the value exp(x)*K(1,x)
  !
  ! NOTES
  ! http://www.advanpix.com/2016/01/05/rational-approximations-for-the-modified-bessel-function-of-the-second-kind-k1-for-computations-with-double-precision/
  !****************************************************************************
  elemental real function BesselKexp1(x)
    real, intent(in) :: x

    real :: t, I1

    if (x <= 0.0) then
       BesselKexp1 = 0.0

    else if (x < 1.0) then
       t = (x)**2

       BesselKexp1 = -3.0796575782920622440538935e-01 &
            + t*(-8.5370719728650778045782736e-02 &
            + t*(-4.6421827664715603298154971e-03 &
            + t*(-1.1253607036630425931072996e-04 &
            + t*(-1.5592887702110907110292728e-06 &
            + t*(-1.4030163679125934402498239e-08 &
            + t*(-8.8718998640336832196558868e-11 &
            + t*(-4.1614323580221539328960335e-13 &
            + t*(-1.5261293392975541707230366e-15 &
            ))))))))

       t = (x/2)**2
       I1 = 8.3333333333333325191635191e-02 &
            + t*(6.9444444444467956461838830e-03 &
            + t*(3.4722222211230452695165215e-04 &
            + t*(1.1574075952009842696580084e-05 &
            + t*(2.7555870002088181016676934e-07 &
            + t*(4.9724386164128529514040614e-09 &
            )))))

       I1 = (1.0 + t*(0.5 + t*I1)) * x/2

       BesselKexp1 = (x*BesselKexp1 + 1/x + log(x)*I1)*exp(x)

    else

       t = 1/x

       BesselKexp1 = 1.0234817795732426171122752e-01 &
            + t*(9.4576473594736724815742878e-01 &
            + t*(2.1876721356881381470401990e+00 &
            + t*(6.0143447861316538915034873e-01 &
            + t*(-1.3961391456741388991743381e-01 &
            + t*(8.8229427272346799004782764e-02 &
            + t*(-8.5494054051512748665954180e-02 &
            + t*(1.0617946033429943924055318e-01 &
            + t*(-1.5284482951051872048173726e-01 &
            + t*(2.3707700686462639842005570e-01 &
            + t*(-3.7345723872158017497895685e-01 &
            + t*(5.6874783855986054797640277e-01 &
            + t*(-8.0418742944483208700659463e-01 &
            + t*(1.0215105768084562101457969e+00 &
            + t*(-1.1342221242815914077805587e+00 &
            + t*(1.0746932686976675016706662e+00 &
            + t*(-8.4904532475797772009120500e-01 &
            + t*(5.4542251056566299656460363e-01 &
            + t*(-2.7630896752209862007904214e-01 &
            + t*(1.0585982409547307546052147e-01 &
            + t*(-2.8751691985417886721803220e-02 &
            + t*(4.9233441525877381700355793e-03 &
            + t*(-3.9900679319457222207987456e-04 &
            ))))))))))))))))))))))

       BesselKexp1 = BesselKexp1 / ( &
            + 8.1662031018453173425764707e-02 &
            + t*(7.2398781933228355889996920e-01 &
            + t*(1.4835841581744134589980018e+00 &
            )))  / sqrt(x)

    end if

  end function BesselKexp1

  !****************************************************************************
  !****f* besselK/BesselKexp2
  ! NAME
  ! real function BesselKexp2(x)
  ! PURPOSE
  ! return the value exp(x)*K(2,x)
  !****************************************************************************
  elemental real function BesselKexp2(x)
    real, intent(in) :: x

    if (x <= 0.0) then
       BesselKexp2 = 0.0
    else
       BesselKexp2 = BesselKexp0(x) + (2.0/x) * BesselKexp1(x)
    end if

  end function BesselKexp2

end module besselK
